/*
 * Copyright (C) 2018 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 * 
**/
#include "inputdialog.h"
#include "ui_inputdialog.h"
#include <QFile>
#include "ukuistylehelper/ukuistylehelper.h"
#include "windowmanager/windowmanager.h"

InputDialog::InputDialog(QStringList names, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::InputDialog)
{
    ui->setupUi(this);
    this->setAutoFillBackground(true);
    this->setBackgroundRole(QPalette::Base);
//    setWindowFlags(/*Qt::FramelessWindowHint |*/ Qt::Dialog);
    QFile qssFile(":/css/assets/promptdialog.qss");
    qssFile.open(QFile::ReadOnly);
    QString styleSheet = QLatin1String(qssFile.readAll());
    this->setStyleSheet(qssFile.readAll());
    qssFile.close();

    //ui->btnClose->setIcon(QIcon(":/images/assets/close.png"));
    ui->btnClose->setProperty("isWindowButton", 0x2);
    ui->btnClose->setProperty("useIconHighlightEffect", 0x8);
    //ui->btnClose->setProperty("setIconHighlightEffectDefaultColor", ui->btnClose->palette().color(QPalette::Active, QPalette::Base));
    ui->btnClose->setFixedSize(30, 30);
    ui->btnClose->setIconSize(QSize(16, 16));
    ui->btnClose->setIcon(QIcon::fromTheme("window-close-symbolic"));
    ui->btnClose->setFlat(true);
    ui->btnClose->setToolTip(tr("Close"));
    ui->btnOK->setEnabled(false);
    ui->lineEdit->setContextMenuPolicy(Qt::NoContextMenu);
    ui->lineEdit->setFocus();
    connect(ui->lineEdit, &QLineEdit::textChanged, this,[=](const QString &text){
        if (text.length() > 64) {
            ui->lblError->setText(tr("feature name is too long"));
        } else if (names.contains(text)){
            ui->lblError->setText(tr("Duplicate feature name"));
        } else if(text.startsWith(" ") || text.endsWith(" ")){
            ui->lblError->setText(tr("Please do not use spaces as the beginning or end of the feature name"));
        } else if(text.isEmpty()) {
            ui->lblError->setText(tr("Empty feature name"));
        } else {
            ui->lblError->setText(tr(""));
        }
        if (!text.isEmpty() && ui->lblError->text().isEmpty()){
            ui->btnOK->setEnabled(true);
        } else {
            ui->btnOK->setEnabled(false);
        }
    });
//    ui->lblError->setStyleSheet("color:red");

    //去除窗管标题栏
    kdk::UkuiStyleHelper::self()->removeHeader(this);
}

InputDialog::~InputDialog()
{
    delete ui;
}

void InputDialog::keyPressEvent(QKeyEvent * event){
    switch (event->key())
    {
    case Qt::Key_Escape:
        break;
    case Qt::Key_Enter:
        if (ui->btnOK->isEnabled())
            ui->btnOK->clicked();
        break;
    case Qt::Key_Return:
        if (ui->btnOK->isEnabled())
            ui->btnOK->clicked();
        break;
    default:
        QDialog::keyPressEvent(event);
    }
}

void InputDialog::setTitle(const QString &text)
{
    //ui->lblTitle->setText(text);
    setWindowTitle(text);
}

void InputDialog::setPrompt(const QString &text)
{
    ui->lblPrompt->setText(text);
}

void InputDialog::setError(const QString &text)
{
    ui->lblError->setText(text);
}

void InputDialog::setText(const QString &text)
{
    ui->lineEdit->setText(text);
}

QString InputDialog::getText()
{
    return ui->lineEdit->text();
}

void InputDialog::on_btnClose_clicked()
{
    reject();
}

void InputDialog::on_btnOK_clicked()
{
    Q_EMIT dataChanged(ui->lineEdit->text());
}

void InputDialog::on_btnCancel_clicked()
{
    reject();
}

//void InputDialog::on_lineEdit_returnPressed()
//{
//    if(ui->lblError->text().isEmpty())
//        Q_EMIT dataChanged(ui->lineEdit->text());
//}
