/*
 * Copyright (C) 2018 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 * 
**/
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QDBusArgument>
#include <QFile>
#include <QProcessEnvironment>
#include <QScrollBar>
#include <QMessageBox>
#include <QKeyEvent>
#include <QPainter>
#include <QMenu>
#include <QDBusInterface>
#include <QRadioButton>
#include <QSettings>
#include <QGSettings>
#include <unistd.h>
#include <pwd.h>
#include <QToolTip>
#include <KWindowSystem>
#include <QPainterPath>
#include <QPainter>
#include "contentpane.h"
#include "customtype.h"
#include "messagedialog.h"
#include "aboutdialog.h"
#include "configuration.h"
#include "attributewindow.h"
#include "kalabel.h"
#include "loadingdialog.h"
//#include "highlight-effect.h"

#define ICON_SIZE 32
#define ICON_TYPE_SCHENA "org.ukui.style"
#define FONT_SIZE "system-font-size"
MainWindow::MainWindow(QString usernameFromCmd, QWidget *parent) :
	QMainWindow(parent),
	ui(new Ui::MainWindow),
    username(usernameFromCmd),
    verificationStatus(false),
    loginSwitch(false),
    saverSwitch(false),
    systemSwitch(false),
    dragWindow(false),
    mWatcher(nullptr),
    lastDeviceInfo(nullptr),
    aboutDlg(nullptr),
    m_uniAuthService(new UniAuthService(this))
{
    setAttribute(Qt::WA_TranslucentBackground);
    connect(WindowManager::self(),&WindowManager::windowAdded,this,[=](const WindowId& windowId){
        /*注意：
         * 最新创建的窗体被设置为操作窗体，此demo中每个按钮对应一个接口函数调用，所有接口函数操作的窗口都是该最新创建的窗体
         */
        if (getpid() == WindowManager::getPid(windowId)) {
            m_listWinIds.append(windowId);
        }
    });
    connect(WindowManager::self(),&WindowManager::windowRemoved,this,[=](const WindowId& windowId){
        if (m_listWinIds.contains(windowId)) {
            m_listWinIds.removeOne(windowId);
        }
    });

    ui->setupUi(this);

    prettify();

    initialize();

//    QFileSystemWatcher *mWatcher = new QFileSystemWatcher(this);
//    mWatcher->addPath(Configuration::configFile);
//    connect(mWatcher,&QFileSystemWatcher::fileChanged,this,[=](const QString &path){
//        mWatcher->addPath(Configuration::configFile);
//        for(int bType = 0; bType < 5; bType++) {
//            if(bType == 0) {
//                QString fp_deviceName = m_bioService->getDefaultDevice(bType);
//                if(fp_defaultName != fp_deviceName) {
//                    Configuration::instance()->defaultDeviceChanged(bType, fp_deviceName);
//                    fp_defaultName = fp_deviceName;
//                }
//            } else if(bType == 1) {
//                QString fc_deviceName = m_bioService->getDefaultDevice(bType);
//                if(fc_defaultName != fc_deviceName) {
//                    Configuration::instance()->defaultDeviceChanged(bType, fc_deviceName);
//                    fc_defaultName = fc_deviceName;
//                }
//            }
//        }
//    });

    if(QGSettings::isSchemaInstalled(ICON_TYPE_SCHENA)){
        QGSettings *icon_type = new QGSettings(ICON_TYPE_SCHENA);
        connect(icon_type, &QGSettings::changed,
            this, &MainWindow::onConfigurationChanged);
    }

    m_styleSettings = new QGSettings(ICON_TYPE_SCHENA,"",this);
    m_curFontSize  = m_styleSettings->get("system-font-size").toInt();
    m_curStyle = m_styleSettings->get("styleName").toString();
    connect(m_styleSettings, &QGSettings::changed,
            this, &MainWindow::onConfigurationChanged);
    setCurStyleSheet();
}

MainWindow::~MainWindow()
{
	delete ui;
}

void MainWindow::onConfigurationChanged(QString key)
{
    if(key == "iconThemeName") {
    	ui->lblLogo->setPixmap(QIcon::fromTheme("biometric-manager").pixmap(QSize(24,24)));
    }

    if(key == "systemFontSize"){
        initPerwgtSize();
    }
    if(key == "styleName"){
        m_curStyle = m_styleSettings->get("styleName").toString();
        initPermissionWidget();
        setCurStyleSheet();
    }
}

void MainWindow::initPermissionWidget()
{
    if(m_curStyle == "ukui-dark" || m_curStyle == "ukui-black"){
        permissionLabel->setStyleSheet("border-radius: 8px;border: 1px solid grey;");
        systemLabel->setStyleSheet("border-bottom-left-radius: 8px;border-bottom-right-radius: 8px;border: 1px solid grey;");
        saverLabel->setStyleSheet("border-style:solid;border-left-width: 1px; border-color:grey;border-right-width: 1px;");
        greetLabel->setStyleSheet("border-top-left-radius: 8px;border-top-right-radius: 8px;border: 1px solid grey;");
    } else {
        permissionLabel->setStyleSheet("border-radius: 8px;border: 1px solid #E7E7E7;");
        systemLabel->setStyleSheet("border-bottom-left-radius: 8px;border-bottom-right-radius: 8px;border: 1px solid #E7E7E7;");
        saverLabel->setStyleSheet("border-style:solid;border-left-width: 1px; border-color:#E7E7E7;border-right-width: 1px;");
        greetLabel->setStyleSheet("border-top-left-radius: 8px;border-top-right-radius: 8px;border: 1px solid #E7E7E7;");
    }
    initPerwgtSize();
}

void MainWindow::initPerwgtSize()
{
    ui->labelPermissions->setText(tr("After this function is enabled, it can be used for system authentication, including login, screen lock, system authorization, and Polkit."));
    ui->labelDeviceType->adjustSize();
    ui->labelPermissions_3->adjustSize();
    QTimer::singleShot(0, this, [&,this](){
        ui->PermissionsWidget->setFixedHeight(22 + ui->labelPermissions->height() + ui->labelPermissions_3->height());
        permissionLabel->setFixedSize(ui->PermissionsWidget->size());
        ui->labelPermissions->setFixedWidth(ui->PermissionsWidget->width() - 80);
        systemLabel->setFixedSize(ui->systemWindow->size());
        saverLabel->setFixedSize(ui->saverWindow->size());
        greetLabel->setFixedSize(ui->greeterWindow->size());
    });
}

void MainWindow::setCurStyleSheet()
{
    if (m_curStyle == "ukui-dark" || m_curStyle == "ukui-black") {
        permissionLabel->setStyleSheet("background-color: black; border-radius: 8px;border: 1px solid #404040;");
        systemLabel->setStyleSheet("background-color: black;border-bottom-left-radius: 8px;border-bottom-right-radius: 8px;border: 1px solid #404040;");
        saverLabel->setStyleSheet("background-color: black;border-style:solid;border-left-width: 1px; border-color:#404040;border-right-width: 1px;");
        greetLabel->setStyleSheet("background-color: black;border-top-left-radius: 8px;border-top-right-radius: 8px;border: 1px solid #404040;");
        QPixmap noDeviceIcon;
        noDeviceIcon.load(":/images/assets/dark-no-device.svg");
        ui->lblNoDeviceIFace->setPixmap(noDeviceIcon);
        ui->lblNoDeviceIFingerPrint->setPixmap(noDeviceIcon);
        ui->lblNoDeviceIFingerVein->setPixmap(noDeviceIcon);
        ui->lblNoDeviceIIris->setPixmap(noDeviceIcon);
        ui->lblNoDeviceIVoicePrint->setPixmap(noDeviceIcon);
    } else {
        permissionLabel->setStyleSheet("background-color: white; border-radius: 8px;border: 1px solid #E7E7E7;");
        systemLabel->setStyleSheet("background-color: white;border-bottom-left-radius: 8px;border-bottom-right-radius: 8px;border: 1px solid #E7E7E7;");
        saverLabel->setStyleSheet("background-color: white;border-style:solid;border-left-width: 1px; border-color:#E7E7E7;border-right-width: 1px;");
        greetLabel->setStyleSheet("background-color: white;border-top-left-radius: 8px;border-top-right-radius: 8px;border: 1px solid #E7E7E7;");
        QPixmap noDeviceIcon;
        noDeviceIcon.load(":/images/assets/no-device.svg");
        ui->lblNoDeviceIFace->setPixmap(noDeviceIcon);
        ui->lblNoDeviceIFingerPrint->setPixmap(noDeviceIcon);
        ui->lblNoDeviceIFingerVein->setPixmap(noDeviceIcon);
        ui->lblNoDeviceIIris->setPixmap(noDeviceIcon);
        ui->lblNoDeviceIVoicePrint->setPixmap(noDeviceIcon);
    }
}

void MainWindow::resizeEvent(QResizeEvent *event)
{
    if (isPermissionWidget) {
        initPerwgtSize();
    }
    if (event->oldSize().width() != event->size().width() && isdashBoardWidget) {
        update_label_width();
    }
}

bool MainWindow::eventFilter(QObject *obj, QEvent *event)
{
    if(obj == ui->widgetHeader && !is_tabletmode) {
        if(event->type() == QEvent::MouseButtonDblClick) {
            if (this->windowState() == Qt::WindowMaximized) {
                ui->btnMax->setIcon(QIcon::fromTheme("window-maximize-symbolic"));
                this->showNormal();
                ui->btnMax->setToolTip(tr("Maximize"));
                initPerwgtSize();
            } else {
                ui->btnMax->setIcon(QIcon::fromTheme("window-restore-symbolic"));
                this->showMaximized();
                ui->btnMax->setToolTip(tr("Restore"));
                initPerwgtSize();
            }
        }
    }
    if(obj->objectName() == "labelIcon"){
        if(event->type() == QEvent::ToolTip){
            DeviceInfo *deviceInfo;
            for(auto info : deviceInfosMap[ui->listWidgetDevicesType->currentIndex()]) {
                if(info->device_shortname == obj->property("devicename").toString()) {
                    if(dialog)
                        return false;
                    deviceInfo = info;
                    dialog =new AttributeWindow(deviceInfo, this);
                    int x = QCursor::pos().x();
                    int y = QCursor::pos().y();
                    dialog->setGeometry(x+ 10, y+ 10, dialog->width(), dialog->height());
                    dialog->show();
                    break;
                }

            }
        }
        else if(event->type() == QEvent::Leave){
            if(dialog == nullptr)
                return false;
            dialog->hide();
            dialog->close();
            dialog = nullptr;
        }
    }
//    if(obj == ui->PermissionsWidget){qDebug()<<"jinlaile";
//        if(event->type() == QEvent::Resize){qDebug()<<"jinlaile";

//    }
//    }
    return false;
}



void MainWindow::mousePressEvent(QMouseEvent *event)
{
 /*
    if(event->button() == Qt::LeftButton) {
        dragPos = event->globalPos() - pos();
        dragWindow = true;
    }
 */
}

void MainWindow::mouseMoveEvent(QMouseEvent *event)
{
 /*
    if(dragWindow) {
        move(event->globalPos() - dragPos);
    }
 */   
}

void MainWindow::keyPressEvent(QKeyEvent *event)
{
    if(event->key() == Qt::Key_F1) {
        if(!daemonIsNotRunning()){
            showGuide("biometric-manager");
        }
    }
}

int MainWindow::daemonIsNotRunning()
{
    QString service_name = "com.kylinUserGuide.hotel_" + QString::number(getuid());
    QDBusConnection conn = QDBusConnection::sessionBus();
    if (!conn.isConnected())
        return 0;

    QDBusReply<QString> reply = conn.interface()->call("GetNameOwner", service_name);
    return reply.value() == "";
}


void MainWindow::showGuide(QString appName)
{
    qDebug() << Q_FUNC_INFO << appName;

    QString service_name = "com.kylinUserGuide.hotel_" + QString::number(getuid());

    QDBusInterface *interface = new QDBusInterface(service_name,
                                                       KYLIN_USER_GUIDE_PATH,
                                                       KYLIN_USER_GUIDE_INTERFACE,
                                                       QDBusConnection::sessionBus(),
                                                       this);

    QDBusMessage msg = interface->call(QStringLiteral("showGuide"),"biometric-manager");
}

void MainWindow::mouseReleaseEvent(QMouseEvent */*event*/)
{
    dragWindow = false;
}

void MainWindow::prettify()
{
    setWindowFlags(Qt::WindowCloseButtonHint);
	/* 设置窗口图标 */
    QApplication::setWindowIcon(QIcon::fromTheme("biometric-manager"));
    /* 设置 CSS */
//    QFile qssFile(":/css/assets/mainwindow.qss");
//    qssFile.open(QFile::ReadOnly);
//    QString styleSheet = QLatin1String(qssFile.readAll());
//    this->setStyleSheet(qssFile.readAll());
//    qssFile.close();
    this->resize(960, 640);
    ui->stackedWidgetMain->setAutoFillBackground(true);
    ui->stackedWidgetMain->setBackgroundRole(QPalette::Base);
    ui->widgetHeader->setAutoFillBackground(true);
    ui->widgetHeader->setBackgroundRole(QPalette::Base);
    ui->widgetHeader->installEventFilter(this);
    ui->widget->installEventFilter(this);

    initLeftsideBar();
    onBtnDashBoardClicked();
    on_listWidgetDevicesType_currentIndexChanged(ui->listWidgetDevicesType->currentIndex());

    ui->lblTitle->setText(tr("Biometric Manager"));
    ui->labelBiometric->setText(tr("Biometric"));
    ui->labelBiometric->adjustSize();
    ui->labelDeviceType->setText(tr("Select Device Type"));
    QPixmap noDeviceIcon;
    noDeviceIcon.load(":/images/assets/no-device.svg");
    ui->labelPermissions->setWordWrap(true);
    ui->labelPermissions->setAlignment(Qt::AlignTop);
    ui->labelPermissions->adjustSize();

    ui->lblNoDeviceIFace->setPixmap(noDeviceIcon);
    ui->lblNoDeviceIFingerPrint->setPixmap(noDeviceIcon);
    ui->lblNoDeviceIFingerVein->setPixmap(noDeviceIcon);
    ui->lblNoDeviceIIris->setPixmap(noDeviceIcon);
    ui->lblNoDeviceIVoicePrint->setPixmap(noDeviceIcon);
//    /* Set Icon for each tab on tabwidget */

//    /* Set logo on lblLogo */
    ui->lblLogo->setPixmap(QIcon::fromTheme("biometric-manager").pixmap(QSize(24,24)));

    ui->btnMin->setProperty("isWindowButton", 0x1);
    ui->btnMin->setProperty("useIconHighlightEffect", 0x2);
    ui->btnMin->setAutoRaise(true);
    ui->btnMin->setFixedSize(30, 30);
    ui->btnMin->setIconSize(QSize(16, 16));
    ui->btnMin->setIcon(QIcon::fromTheme("window-minimize-symbolic"));
    ui->btnMin->setToolTip(tr("Minimize"));

    ui->btnMax->setProperty("isWindowButton", 0x1);
    ui->btnMax->setProperty("useIconHighlightEffect", 0x2);
    ui->btnMax->setAutoRaise(true);
    ui->btnMax->setFixedSize(30, 30);
    ui->btnMax->setIconSize(QSize(16, 16));
    ui->btnMax->setIcon(QIcon::fromTheme("window-maximize-symbolic"));
    ui->btnMax->setToolTip(tr("Maximize"));

    ui->tableWidgetDevices->setAlternatingRowColors(true);
    ui->tableWidgetDevices->horizontalHeader()->setSectionResizeMode(QHeaderView::Fixed);

    ui->btnMenu->setProperty("isWindowButton", 0x1);
    ui->btnMenu->setProperty("useIconHighlightEffect", 0x2);
    ui->btnMenu->setAutoRaise(true);
    ui->btnMenu->setFixedSize(30, 30);
    ui->btnMenu->setIconSize(QSize(16, 16));
    ui->btnMenu->setIcon(QIcon::fromTheme("open-menu-symbolic"));
    ui->btnMenu->setToolTip(tr("Options"));

    ui->btnClose->setProperty("isWindowButton", 0x2);
    ui->btnClose->setProperty("useIconHighlightEffect", 0x8);
    ui->btnClose->setAutoRaise(true);
    ui->btnClose->setFixedSize(30, 30);
    ui->btnClose->setIconSize(QSize(16, 16));
    ui->btnClose->setIcon(QIcon::fromTheme("window-close-symbolic"));
    ui->btnClose->setToolTip(tr("Close"));

    ui->tableWidgetDevices->verticalScrollBar()->setContextMenuPolicy(Qt::NoContextMenu);

    permissionLabel = new QLabel(ui->PermissionsWidget);
    permissionLabel->installEventFilter(this);
    permissionLabel->lower();
    systemLabel = new QLabel(ui->systemWindow);
    systemLabel->lower();

    saverLabel = new QLabel(ui->saverWindow);
    saverLabel->lower();

    greetLabel = new QLabel(ui->greeterWindow);
    greetLabel->lower();
    ui->biometricLabel->hide();
    ui->greeterWindow->hide();
    ui->saverWindow->hide();
    ui->systemWindow->hide();
    QVBoxLayout *permissionLyt = new QVBoxLayout(this);
    ui->pagePermissions->setLayout(permissionLyt);
    permissionLyt->addWidget(ui->perWidget);
    initPermissionWidget();

    //监听平板模式切换
    m_statusSessionDbus = new QDBusInterface("com.kylin.statusmanager.interface",
                                              "/",
                                              "com.kylin.statusmanager.interface",
                                              QDBusConnection::sessionBus(),this);
    tabletStatusChanged(is_tabletmode);
    if (m_statusSessionDbus->isValid()) {
        is_tabletmode = m_statusSessionDbus->call("get_current_tabletmode");
        tabletStatusChanged(is_tabletmode);
        connect(m_statusSessionDbus, SIGNAL(mode_change_signal(bool)), this, SLOT(tabletStatusChanged(bool)));
    } else {
        qDebug() << "Create statusmanager Interface Failed When : " << QDBusConnection::systemBus().lastError();
    }
}

void MainWindow::initLeftsideBar()
{
    scrollArea = new KNavigationBar(ui->leftWidget);
    scrollArea->setMinimumHeight(640);

    scrollArea->addTag(tr("Device&Permission"));
    permissionItem = new QStandardItem(QIcon(":/images/assets/Permissions.svg"), tr("Permissions"));
    scrollArea->addItem(permissionItem);

    dashBoardItem = new QStandardItem(QIcon(":/images/assets/Device.svg"), tr("Biometric"));
    scrollArea->addItem(dashBoardItem);

    scrollArea->addTag(tr("Password Setting"));
    fingerPrintItem = new QStandardItem(QIcon(":/images/assets/FingerPrint.svg"), tr("FingerPrint"));
    scrollArea->addItem(fingerPrintItem);
    faceItem = new QStandardItem(QIcon(":/images/assets/faceid.svg"), tr("Face"));
    scrollArea->addItem(faceItem);
    fingerVeinItem = new QStandardItem(QIcon(":/images/assets/FingerVein.svg"), tr("FingerVein"));
    scrollArea->addItem(fingerVeinItem);
    irisItem = new QStandardItem(QIcon(":/images/assets/Iris.svg"), tr("Iris"));
    scrollArea->addItem(irisItem);
    voiceprintItem = new QStandardItem(QIcon(":/images/assets/VoicePrint.svg"), tr("VoicePrint"));
    scrollArea->addItem(voiceprintItem);
    connect(scrollArea->listview(), &QListView::clicked, this, [=](const QModelIndex &index){
        switch (index.row()) {
        case 1:
            onBtnPermissionsClicked();
            break;
        case 2:
            onBtnDashBoardClicked();
            break;
        case 4:
            onBtnFingerPrintClicked();
            break;
        case 5:
            onBtnFaceClicked();
            break;
        case 6:
            onBtnFingerVeinClicked();
            break;
        case 7:
            onBtnIrisClicked();
            break;
        case 8:
            onBtnVoicePrintClicked();
            break;
        default:
            break;
        }
    });
    scrollArea->listview()->setFixedWidth(216);
}

QPixmap *MainWindow::getUserAvatar(QString username)
{
    QString iconPath;
    QDBusInterface userIface( "org.freedesktop.Accounts", "/org/freedesktop/Accounts",
              "org.freedesktop.Accounts", QDBusConnection::systemBus());
    if (!userIface.isValid())
        qDebug() << "GUI:" << "userIface is invalid";
    QDBusReply<QDBusObjectPath> userReply = userIface.call("FindUserByName", username);
    if (!userReply.isValid()) {
        qDebug() << "GUI:" << "userReply is invalid";
        iconPath = "/usr/share/kylin-greeter/default_face.png";
    }
    QDBusInterface iconIface( "org.freedesktop.Accounts", userReply.value().path(),
            "org.freedesktop.DBus.Properties", QDBusConnection::systemBus());
    if (!iconIface.isValid())
        qDebug() << "GUI:" << "IconIface is invalid";
    QDBusReply<QDBusVariant> iconReply = iconIface.call("Get", "org.freedesktop.Accounts.User", "IconFile");
    if (!iconReply.isValid()) {
        qDebug() << "GUI:" << "iconReply is invalid";
        iconPath = "/usr/share/kylin-greeter/default_face.png";
    }
    iconPath = iconReply.value().variant().toString();
    if (access(qPrintable(iconPath), R_OK) != 0) /* No Access Permission */
        iconPath = "/usr/share/kylin-greeter/default_face.png";
    return new QPixmap(iconPath);
}

void MainWindow::setCurrentUser()
{
//    struct passwd *pwd;
//    pwd = getpwuid(getuid());
//    username = QString(pwd->pw_name);
//    ui->lblUserName->setText(username);

//    ui->lblAvatar->setPixmap(QPixmap(":/images/assets/avatar.png"));
}

void MainWindow::onReviceWindowMessage(QString message)
{
    QString platform = QGuiApplication::platformName();
    if(platform.startsWith(QLatin1String("wayland"),Qt::CaseInsensitive)) {
        if (!m_listWinIds.isEmpty()) {
            WindowManager::activateWindow(m_listWinIds.back());
        }
    } else {
        KWindowSystem::forceActiveWindow(this->winId());
    }
    this->show();
}

void MainWindow::initialize()
{
    /* 向 QDBus 类型系统注册自定义数据类型 */
    registerCustomTypes();
    /* 连接 DBus Daemon */
    serviceInterface = new QDBusInterface(DBUS_SERVICE, DBUS_PATH,
                                          DBUS_INTERFACE,
                                          QDBusConnection::systemBus());
    serviceInterface->setTimeout(2147483647); /* 微秒 */

    initSysMenu();

    /* 获取并显示用户 */
    setCurrentUser();

    /* 获取设备列表 */
    getDeviceInfo();

    /* Other initializations */
    initDashboardBioAuthSection();
//    initSaverSwitch();
//    initSystemSwitch();
//    initLoginSwitch();
    initBiometricPage();
    initDeviceTypeList();
    connect(ui->btnMin, &QToolButton::clicked, this, &MainWindow::showMinimized);
    connect(ui->btnMax, &QToolButton::clicked, this, [=] {
            if (isMaximized()) {
                showNormal();
                ui->btnMax->setIcon(QIcon::fromTheme("window-maximize-symbolic"));
                ui->btnMax->setToolTip(tr("Maximize"));
            } else {
                showMaximized();
                ui->btnMax->setIcon(QIcon::fromTheme("window-restore-symbolic"));
                ui->btnMax->setToolTip(tr("Restore"));
            }
        });
    connect(ui->btnClose, &QPushButton::clicked, this, &MainWindow::close);


    ui->btnStatus->setCheckable(true);
    connect(ui->btnStatus, &KSwitchButton::clicked, this, &MainWindow::on_btnStatus_clicked);
    connect(ui->btnSaver, &KSwitchButton::clicked, this, &MainWindow::on_btnSaver_clicked);
    connect(ui->btnLogin, &KSwitchButton::clicked, this, &MainWindow::on_btnLogin_clicked);
    connect(ui->btnSystem, &KSwitchButton::clicked, this, &MainWindow::on_btnSystem_clicked);

    connect(serviceInterface, SIGNAL(USBDeviceHotPlug(int, int, int)),
            this, SLOT(onUSBDeviceHotPlug(int,int,int)));
    connect(serviceInterface, SIGNAL(FeatureChanged(int, int, int)),
            this, SLOT(onUpdateDevice(int,int,int)));
}


void MainWindow::initSysMenu()
{
    menu = new QMenu(this);
    QAction *serviceStatusAction = new QAction(tr("Restart Service"), this);
    connect(serviceStatusAction, &QAction::triggered, this, [&]{
        restartService();
    });

    QAction *aboutAction = new QAction(tr("About"), this);
    connect(aboutAction, &QAction::triggered, this, [&]{
        if(aboutDlg == nullptr)
            aboutDlg = new KAboutDialog(this,QIcon::fromTheme("biometric-manager"), tr("Biometric Manager"));

        int x = this->geometry().topLeft().x() + (width() - aboutDlg->width()) / 2;
        int y = this->geometry().topLeft().y() + (height() - aboutDlg->height()) / 2;
        aboutDlg->setAppVersion(Configuration::instance()->get_current_version());
        aboutDlg->setAppSupport(aboutDlg->appSupport());
        aboutDlg->move(x, y);
        aboutDlg->show();
    });

    QAction *exitAction = new QAction(tr("Exit"), this);
    connect(exitAction, &QAction::triggered, this, [&]{
        close();
    });

    QAction *helpAction = new QAction(tr("Help"), this);
    connect(helpAction, &QAction::triggered, this, [&]{
        if(!daemonIsNotRunning()){
            showGuide("biometric-manager");
        }
    });

    menu->addActions({serviceStatusAction, helpAction,aboutAction,exitAction});
    ui->btnMenu->setPopupMode(QToolButton::InstantPopup   );
    ui->btnMenu->setMenu(menu);
}

void MainWindow::onBtnDashBoardClicked()
{
    ui->stackedWidgetMain->setCurrentWidget(ui->pageDashBoard);
    isdashBoardWidget = true;
    isPermissionWidget = false;
    scrollArea->listview()->setCurrentIndex(dashBoardItem->index());
}

void MainWindow::onBtnPermissionsClicked()
{
    ui->stackedWidgetMain->setCurrentWidget(ui->pagePermissions);
    isPermissionWidget = true;
    isdashBoardWidget = false;
    scrollArea->listview()->setCurrentIndex(permissionItem->index());
    initPerwgtSize();
}

void MainWindow::onBtnFingerPrintClicked()
{
    ui->stackedWidgetMain->setCurrentWidget(ui->pageFingerPrint);
    isPermissionWidget = false;
    isdashBoardWidget = false;
    scrollArea->listview()->setCurrentIndex(fingerPrintItem->index());
}

void MainWindow::onBtnFingerVeinClicked()
{
    ui->stackedWidgetMain->setCurrentWidget(ui->pageFingerVein);
    isdashBoardWidget = false;
    isPermissionWidget = false;
    scrollArea->listview()->setCurrentIndex(fingerVeinItem->index());
}

void MainWindow::onBtnIrisClicked()
{
    ui->stackedWidgetMain->setCurrentWidget(ui->pageIris);
    isPermissionWidget = false;
    isdashBoardWidget = false;
    scrollArea->listview()->setCurrentIndex(irisItem->index());
}

void MainWindow::onBtnVoicePrintClicked()
{
    ui->stackedWidgetMain->setCurrentWidget(ui->pageVoicePrint);
    isPermissionWidget = false;
    isdashBoardWidget = false;
    scrollArea->listview()->setCurrentIndex(voiceprintItem->index());
}

void MainWindow::onBtnFaceClicked()
{
    ui->stackedWidgetMain->setCurrentWidget(ui->pageFace);
    isPermissionWidget = false;
    isdashBoardWidget = false;
    scrollArea->listview()->setCurrentIndex(faceItem->index());
}

void MainWindow::on_lblNoDevicePFingerPrint_clicked()
{
    setStackedWidgetMain(ui->pageDashBoard);
    ui->listWidgetDevicesType->setCurrentIndex(0);
}

void MainWindow::on_lblNoDevicePFace_clicked()
{
    setStackedWidgetMain(ui->pageDashBoard);
    ui->listWidgetDevicesType->setCurrentIndex(1);
}

void MainWindow::on_lblNoDevicePFingerVein_clicked()
{
    setStackedWidgetMain(ui->pageDashBoard);
    ui->listWidgetDevicesType->setCurrentIndex(2);
}

void MainWindow::on_lblNoDevicePIris_clicked()
{
    setStackedWidgetMain(ui->pageDashBoard);
    ui->listWidgetDevicesType->setCurrentIndex(3);
}

void MainWindow::on_lblNoDevicePVoicePrint_clicked()
{
    setStackedWidgetMain(ui->pageDashBoard);
    ui->listWidgetDevicesType->setCurrentIndex(4);
}

/**
 * @brief 设备类型到索引的映射
 */
int MainWindow::bioTypeToIndex(int type)
{
    switch(type) {
    case BIOTYPE_FINGERPRINT:
        return 0;
    case BIOTYPE_FACE:
        return 1;
    case BIOTYPE_FINGERVEIN:
        return 2;
    case BIOTYPE_IRIS:
        return 3;
    case BIOTYPE_VOICEPRINT:
        return 4;
    }
    return -1;
}

void MainWindow::updateDeviceNum(int type)
{
    switch (type) {
    case BIOTYPE_FINGERPRINT:
        fpNum += 1;
        break;
    case BIOTYPE_FACE:
        fcNum += 1;
        break;
    case BIOTYPE_FINGERVEIN:
        fvNum += 1;
        break;
    case BIOTYPE_IRIS:
        irNum += 1;
        break;
    case BIOTYPE_VOICEPRINT:
        vpNum += 1;
        break;
    default:
        break;
    }
    return ;
}

int MainWindow::getDeviceNum(int type)
{
    switch(type) {
    case BIOTYPE_FINGERPRINT:
        return fpNum;
    case BIOTYPE_FACE:
        return fcNum;
    case BIOTYPE_FINGERVEIN:
        return fvNum;
    case BIOTYPE_IRIS:
        return irNum;
    case BIOTYPE_VOICEPRINT:
        return vpNum;
    }
    return 0;
}

void MainWindow::clearDeviceNum()
{
    fpNum = 0;
    fcNum = 0;
    fvNum = 0;
    irNum = 0;
    vpNum = 0;
}

/**
 * @brief 获取设备列表并存储起来备用
 */
void MainWindow::getDeviceInfo()
{
	QVariant variant;
	QDBusArgument argument;
	QList<QDBusVariant> qlist;
	QDBusVariant item;
	DeviceInfo *deviceInfo;

	/* 返回值为 i -- int 和 av -- array of variant */
    QDBusPendingReply<int, QList<QDBusVariant> > reply = serviceInterface->call("GetDrvList");
	reply.waitForFinished();
	if (reply.isError()) {
		qDebug() << "GUI:" << reply.error();
		deviceCount = 0;
		return;
	}

	/* 解析 DBus 返回值，reply 有两个返回值，都是 QVariant 类型 */
	variant = reply.argumentAt(0); /* 得到第一个返回值 */
	deviceCount = variant.value<int>(); /* 解封装得到设备个数 */
	variant = reply.argumentAt(1); /* 得到第二个返回值 */
	argument = variant.value<QDBusArgument>(); /* 解封装，获取QDBusArgument对象 */
	argument >> qlist; /* 使用运算符重载提取 argument 对象里面存储的列表对象 */

    for(int i = 0; i < __MAX_NR_BIOTYPES; i++)
        deviceInfosMap[i].clear();;

	for (int i = 0; i < deviceCount; i++) {
		item = qlist[i]; /* 取出一个元素 */
		variant = item.variant(); /* 转为普通QVariant对象 */
		/* 解封装得到 QDBusArgument 对象 */
		argument = variant.value<QDBusArgument>();
		deviceInfo = new DeviceInfo();
		argument >> *deviceInfo; /* 提取最终的 DeviceInfo 结构体 */

        deviceInfosMap[bioTypeToIndex(deviceInfo->biotype)].append(deviceInfo);
        if(deviceInfo->device_available > 0)
            updateDeviceNum(deviceInfo->biotype);
        //qDebug() << deviceInfo->biotype << deviceInfo->device_shortname << deviceInfo->device_available << deviceCount;
	}
}

void MainWindow::setLastDeviceSelected()
{
    if(!lastDeviceInfo)
        return ;

    QComboBox *lw;
    QStackedWidget *sw;

    switch(lastDeviceInfo->biotype) {
    case BIOTYPE_FINGERPRINT:
        lw = ui->listWidgetFingerPrint;
        sw = ui->stackedWidgetFingerPrint;
        break;
    case BIOTYPE_FINGERVEIN:
        lw = ui->listWidgetFingerVein;
        sw = ui->stackedWidgetFingerVein;
        break;
    case BIOTYPE_IRIS:
        lw = ui->listWidgetIris;
        sw = ui->stackedWidgetIris;
        break;
    case BIOTYPE_FACE:
        lw = ui->listWidgetFace;
        sw = ui->stackedWidgetFace;
        break;
    case BIOTYPE_VOICEPRINT:
        lw = ui->listWidgetVoicePrint;
        sw = ui->stackedWidgetVoicePrint;
        break;
    }

    lastDeviceInfo = nullptr;

    for(int i=0;i<lw->count();i++)
    {
        if(lw->currentText()==lastDeviceInfo->device_shortname)
        {
            lw->setCurrentIndex(i);
            sw->setCurrentIndex(i);
        }
    }

    lastDeviceInfo = nullptr;
}

void MainWindow::raiseContentPane(DeviceInfo *deviceInfo)
{
    if(deviceInfo->device_available<=0)
        return ;

    QComboBox *lw;
    QStackedWidget *sw;
    switch(deviceInfo->biotype) {
    case BIOTYPE_FINGERPRINT:
        lw = ui->listWidgetFingerPrint;
        sw = ui->stackedWidgetFingerPrint;
        break;
    case BIOTYPE_FINGERVEIN:
        lw = ui->listWidgetFingerVein;
        sw = ui->stackedWidgetFingerVein;
        break;
    case BIOTYPE_IRIS:
        lw = ui->listWidgetIris;
        sw = ui->stackedWidgetIris;
        break;
    case BIOTYPE_FACE:
        lw = ui->listWidgetFace;
        sw = ui->stackedWidgetFace;
        break;
    case BIOTYPE_VOICEPRINT:
        lw = ui->listWidgetVoicePrint;
        sw = ui->stackedWidgetVoicePrint;
        break;
    }
    for(int i=0;i<lw->count();i++)
    {
        if(lw->itemText(i) == deviceInfo->device_shortname)
        {
            //lw->addItem(deviceInfo->device_shortname);
            QWidget *widget = sw->widget(i);
            sw->removeWidget(sw->widget(i));
            sw->insertWidget(0,widget);
        }
    }

//    for(int i=0;i<lw->count();i++)
//    {
//        if(lw->item(i)->text()==deviceInfo->device_shortname)
//        {
//            QListWidgetItem *item = lw->takeItem(i);
//            lw->insertItem(0,item);
//            QWidget *widget = sw->widget(i);
//            sw->removeWidget(sw->widget(i));
//            sw->insertWidget(0,widget);
//        }
//    }

}

void MainWindow::addContentPane(DeviceInfo *deviceInfo)
{
    QComboBox *lw;
    QStackedWidget *sw;

    switch(deviceInfo->biotype) {
    case BIOTYPE_FINGERPRINT:
        lw = ui->listWidgetFingerPrint;
        sw = ui->stackedWidgetFingerPrint;
        break;
    case BIOTYPE_FINGERVEIN:
        lw = ui->listWidgetFingerVein;
        sw = ui->stackedWidgetFingerVein;
        break;
    case BIOTYPE_IRIS:
        lw = ui->listWidgetIris;
        sw = ui->stackedWidgetIris;
        break;
    case BIOTYPE_FACE:
        lw = ui->listWidgetFace;
        sw = ui->stackedWidgetFace;
        break;
    case BIOTYPE_VOICEPRINT:
        lw = ui->listWidgetVoicePrint;
        sw = ui->stackedWidgetVoicePrint;
        break;
    }

//    QListWidgetItem *item = new QListWidgetItem(deviceInfo->device_shortname);
//    ContentPane *contentPane = new ContentPane(getuid(), deviceInfo);
//    item->setTextAlignment(Qt::AlignCenter);
//    if(deviceInfo->device_available <= 0){
//        lw->insertItem(lw->count(), item);
//        sw->insertWidget(sw->count(),contentPane);
//    }
//    else {
//        lw->insertItem(0,item);
//        sw->insertWidget(0,contentPane);
//    }
//    if(deviceInfo->device_available <= 0)
//        item->setTextColor(Qt::gray);

//    contentPaneMap.insert(deviceInfo->device_shortname, contentPane);

    ContentPane *contentPane = new ContentPane(getuid(), deviceInfo);
    connect(contentPane, &ContentPane::deleteFeatureState, this, &MainWindow::onDeleteFeatureState);
    QString default_Name = m_uniAuthService->getDefaultDevice(getenv("USER"), deviceInfo->biotype);
    QString defaultText = tr("(default) ");
    defaultText = defaultText + deviceInfo->device_shortname;
    if(deviceInfo->device_available > 0){
        if(deviceInfo->device_shortname == default_Name)
            lw->addItem(defaultText);
        else
            lw->addItem(deviceInfo->device_shortname);
        sw->insertWidget(sw->count(),contentPane);
    }
//    else {
//        lw->insertItem(0,deviceInfo->device_shortname);
//        sw->insertWidget(0,contentPane);
//    }
//    if(deviceInfo->device_available <= 0)
//        item->setTextColor(Qt::gray);

    contentPaneMap.insert(deviceInfo->device_shortname, contentPane);

    //connect(lw, &QListWidget::currentRowChanged, sw, &QStackedWidget::setCurrentIndex);
    connect(lw, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged), sw, &QStackedWidget::setCurrentIndex);
//    connect(contentPane, &ContentPane::changeDeviceStatus, this, &MainWindow::changeDeviceStatus);
}

#define checkBiometricPage(biometric) do {				\
    if (ui->listWidget##biometric->count() >= 1) {			\
        ui->listWidget##biometric->setCurrentIndex(0);		\
		ui->listWidget##biometric->show();			\
        ui->listWidgetA##biometric->show();			\
        ui->listWidgetB##biometric->show();			\
		ui->stackedWidget##biometric->show();			\
        ui->lblNoDeviceW##biometric->hide();			\
	} else {							\
		ui->listWidget##biometric->hide();			\
        ui->listWidgetA##biometric->hide();			\
        ui->listWidgetB##biometric->hide();			\
		ui->stackedWidget##biometric->hide();			\
        ui->lblNoDeviceW##biometric->show();			\
	}								\
} while(0)

void MainWindow::updateBiometricPage(int type ,int devNumNow)
{
    if(type == 0){
        if(devNumNow == 0){
            ui->listWidgetFingerPrint->hide();
            ui->stackedWidgetFingerPrint->hide();
            ui->listWidgetAFingerPrint->hide();
            ui->listWidgetBFingerPrint->hide();
            ui->lblNoDeviceWFingerPrint->show();
        }
        else{
            ui->listWidgetFingerPrint->show();
            ui->stackedWidgetFingerPrint->show();
            ui->listWidgetAFingerPrint->show();
            ui->listWidgetBFingerPrint->show();
            ui->lblNoDeviceWFingerPrint->hide();
        }
    }
    if(type == 1){
        if(devNumNow == 0){
            ui->listWidgetFace->hide();
            ui->stackedWidgetFace->hide();
            ui->listWidgetAFace->hide();
            ui->listWidgetBFace->hide();
            ui->lblNoDeviceWFace->show();
        }
        else{
            ui->listWidgetFace->show();
            ui->stackedWidgetFace->show();
            ui->listWidgetAFace->show();
            ui->listWidgetBFace->show();
            ui->lblNoDeviceWFace->hide();
        }
    }
    if(type == 2){
        if(devNumNow == 0){
            ui->listWidgetFingerVein->hide();
            ui->stackedWidgetFingerVein->hide();
            ui->listWidgetAFingerVein->hide();
            ui->listWidgetBFingerVein->hide();
            ui->lblNoDeviceWFingerVein->show();
            }
        else{
            ui->listWidgetFingerVein->show();
            ui->stackedWidgetFingerVein->show();
            ui->listWidgetAFingerVein->show();
            ui->listWidgetBFingerVein->show();
            ui->lblNoDeviceWFingerVein->hide();
        }
    }
    if(type == 3){
        if(devNumNow == 0){
            ui->listWidgetIris->hide();
            ui->stackedWidgetIris->hide();
            ui->listWidgetAIris->hide();
            ui->listWidgetBIris->hide();
            ui->lblNoDeviceWIris->show();
        }
        else{
            ui->listWidgetIris->show();
            ui->stackedWidgetIris->show();
            ui->listWidgetAIris->show();
            ui->listWidgetBIris->show();
            ui->lblNoDeviceWIris->hide();
        }
    }
    if(type == 4){
        if(devNumNow == 0){
            ui->listWidgetVoicePrint->hide();
            ui->stackedWidgetVoicePrint->hide();
            ui->listWidgetAVoicePrint->hide();
            ui->listWidgetBVoicePrint->hide();
            ui->lblNoDeviceWVoicePrint->show();
        }
        else{
            ui->listWidgetVoicePrint->show();
            ui->stackedWidgetVoicePrint->show();
            ui->listWidgetAVoicePrint->show();
            ui->listWidgetBVoicePrint->show();
            ui->lblNoDeviceWVoicePrint->hide();
        }
    }
}

void MainWindow::initBiometricPage()
{
    ui->listWidgetFingerPrint->clear();
    for(int i = ui->stackedWidgetFingerPrint->count(); i >= 0; i--)
    {
        QWidget* widget = ui->stackedWidgetFingerPrint->widget(i);
        ui->stackedWidgetFingerPrint->removeWidget(widget);
        if (widget) {
            widget->close();
            widget->deleteLater();
        }
    }
    ui->listWidgetFingerVein->clear();
    for(int i = ui->stackedWidgetFingerVein->count(); i >= 0; i--)
    {
        QWidget* widget = ui->stackedWidgetFingerVein->widget(i);
        ui->stackedWidgetFingerVein->removeWidget(widget);
        if (widget) {
            widget->close();
            widget->deleteLater();
        }
    }
    ui->listWidgetIris->clear();
    for(int i = ui->stackedWidgetIris->count(); i >= 0; i--)
    {
        QWidget* widget = ui->stackedWidgetIris->widget(i);
        ui->stackedWidgetIris->removeWidget(widget);
        if (widget) {
            widget->close();
            widget->deleteLater();
        }
    }
    ui->listWidgetVoicePrint->clear();
    for(int i = ui->stackedWidgetVoicePrint->count(); i >= 0; i--)
    {
        QWidget* widget = ui->stackedWidgetVoicePrint->widget(i);
        ui->stackedWidgetVoicePrint->removeWidget(widget);
        if (widget) {
            widget->close();
            widget->deleteLater();
        }
    }
    ui->listWidgetFace->clear();
    for(int i = ui->stackedWidgetFace->count(); i >= 0; i--)
    {
        QWidget* widget = ui->stackedWidgetFace->widget(i);
        ui->stackedWidgetFace->removeWidget(widget);
        if (widget) {
            widget->close();
            widget->deleteLater();
        }
    }
    contentPaneMap.clear();

    for(int i = 0; i < __MAX_NR_BIOTYPES; i++){
        for (auto deviceInfo : deviceInfosMap[i]){
            addContentPane(deviceInfo);
        }
    }
    checkBiometricPage(FingerPrint);
    checkBiometricPage(FingerVein);
    checkBiometricPage(Iris);
    checkBiometricPage(VoicePrint);
    checkBiometricPage(Face);
}

void MainWindow::onDeleteFeatureState(int nState)
{
    switch(nState){
    case 0: // 开始删除
        if (!m_isShowDeleteAnimation) {
            m_isShowDeleteAnimation = true;
            if (!m_dialogLoading) {
                m_dialogLoading = new LoadingDialog(this, 0.4);
                m_dialogLoading->setGeometry(0,0, this->geometry().width(), this->geometry().height());

            } else {
                m_dialogLoading->setLoading();
            }
            m_dialogLoading->show();
        }
        break;
    case 1: // 删除完成
        if (m_isShowDeleteAnimation) {
            m_isShowDeleteAnimation = false;
            if (m_dialogLoading) {
                m_dialogLoading->hide();
                m_dialogLoading->close();
            }
        }
        break;
    }
}

void MainWindow::sortContentPane()
{
    for(int i = 0; i < __MAX_NR_BIOTYPES; i++)
        for (auto deviceInfo : deviceInfosMap[i])
            raiseContentPane(deviceInfo);
    checkBiometricPage(FingerPrint);
    checkBiometricPage(FingerVein);
    checkBiometricPage(Iris);
    checkBiometricPage(VoicePrint);
    checkBiometricPage(Face);
//    if(lastDeviceInfo)
//        setLastDeviceSelected();
}

#define SET_TABLE_ATTRIBUTE(tw) do {					\
	tw->setColumnCount(2);						\
	tw->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);\
	tw->setHorizontalHeaderLabels(QStringList()			\
		<< QString(tr("Device Name")) << QString(tr("Status")));\
	tw->verticalHeader()->setVisible(false);			\
	tw->horizontalHeader()->setSectionResizeMode(1,			\
					QHeaderView::ResizeToContents);	\
	tw->setSelectionMode(QAbstractItemView::NoSelection);		\
} while (0);



void MainWindow::initDashboardBioAuthSection()
{
    connect(m_uniAuthService, &UniAuthService::bioAuthStatusChanged, this, &MainWindow::setVerificationStatus);
    for(int i = 0; i <= 3; i++) {
        setVerificationStatus(getenv("USER"), i, m_uniAuthService->getBioAuthStatus(getenv("USER"), i));
    }
}

void MainWindow::initLoginSwitch()
{
    if(loginSwitch)
        setLoginSwitch(true);
    else
        setLoginSwitch(false);
}

void MainWindow::initSaverSwitch()
{
    if(saverSwitch)
        setSaverSwitch(true);
    else
        setSaverSwitch(false);
}

void MainWindow::initSystemSwitch()
{
    if(systemSwitch)
        setSystemSwitch(true);
    else
        setSystemSwitch(false);
}

void MainWindow::initDeviceTypeList()
{
    QStringList devicesTypeText = {tr("FingerPrint"),  tr("Face"), tr("FingerVein"),
                                   tr("Iris"), tr("VoicePrint")};
    for(int i = 0; i < devicesTypeText.size(); i++)
        ui->listWidgetDevicesType->insertItem(ui->listWidgetDevicesType->count(), devicesTypeText[i]);

    ui->listWidgetDevicesType->setCurrentIndex(0);
}


void MainWindow::setVerificationStatus(QString userName, int type, bool status)
{
    if(userName != getenv("USER"))
        return ;
    switch (type) {
    case MAIN_SWITCH:
        verificationStatus = status;
        if (status) {
             ui->biometricLabel->show();
             ui->greeterWindow->show();
             ui->saverWindow->show();
             ui->systemWindow->show();
             ui->btnStatus->setChecked(true);
        }
        else {
            ui->biometricLabel->hide();
            ui->greeterWindow->hide();
            ui->saverWindow->hide();
            ui->systemWindow->hide();
            ui->btnStatus->setChecked(false);
        }
        break;
    case SAVER_SWITCH:
        saverSwitch = status;
        if (status) {
            ui->btnSaver->setChecked(true);
        }
        else {
            ui->btnSaver->setChecked(false);
        }
        break;
    case GREETER_SWITCH:
        loginSwitch = status;
        if (status) {
            ui->btnLogin->setChecked(true);
        }
        else {
            ui->btnLogin->setChecked(false);
        }
        break;
    case POLKIT_SWITCH:
        systemSwitch = status;
        if (status) {
            ui->btnSystem->setChecked(true);
        }
        else {
            ui->btnSystem->setChecked(false);
        }
        break;
    default:
        break;
    }
}

void MainWindow::on_btnStatus_clicked()
{
    if (verificationStatus) {
        m_uniAuthService->setBioAuthStatus(MAIN_SWITCH, false);
    } else {
        m_uniAuthService->setBioAuthStatus(MAIN_SWITCH, true);
        initPerwgtSize();
    }
}

void MainWindow::on_btnLogin_clicked()
{
    if (loginSwitch) {
        m_uniAuthService->setBioAuthStatus(GREETER_SWITCH, false);
    } else {
        m_uniAuthService->setBioAuthStatus(GREETER_SWITCH, true);
    }
}

void MainWindow::on_btnSaver_clicked()
{
    if (saverSwitch) {
        m_uniAuthService->setBioAuthStatus(SAVER_SWITCH, false);
    } else {
        m_uniAuthService->setBioAuthStatus(SAVER_SWITCH, true);
    }
}

void MainWindow::on_btnSystem_clicked()
{
    if (systemSwitch) {
        m_uniAuthService->setBioAuthStatus(POLKIT_SWITCH, false);
    } else {
        m_uniAuthService->setBioAuthStatus(POLKIT_SWITCH, true);
    }
}

void MainWindow::setSaverSwitch(bool status)
{
   saverSwitch = status;
   if (status) {
       ui->btnSaver->setChecked(true);
   }
   else {
       ui->btnSaver->setChecked(false);
   }
}

void MainWindow::setLoginSwitch(bool status)
{
   loginSwitch = status;
   if (status) {
       ui->btnLogin->setChecked(true);
   }
   else {
       ui->btnLogin->setChecked(false);
   }
}

void MainWindow::setSystemSwitch(bool status)
{
   systemSwitch = status;
   if (status) {
       ui->btnSystem->setChecked(true);
   }
   else {
       ui->btnSystem->setChecked(false);
   }
}

void MainWindow::onDefaultDeviceChanged(bool checked)
{
    //如果选中的是默认设备，直接return(每种设备类型至少有一个默认设备)
    if(!checked) {
        for(auto cb : btnGroup)
            if(cb == sender())
                cb->setChecked(true);
        return;
    }
    QString objName = sender()->objectName();
    int spliter = objName.lastIndexOf('-');
    QString deviceName = objName.left(spliter);
    int deviceType = objName.right(objName.length() - spliter - 1).toInt();
    for(auto cb : btnGroup)
        if(cb != sender())
            cb->setChecked(false);

    if(checked)
        m_uniAuthService->setDefaultDevice(deviceType, deviceName);
    else
        m_uniAuthService->setDefaultDevice(deviceType, "");
    on_listWidgetDevicesType_currentIndexChanged(ui->listWidgetDevicesType->currentIndex());
    initBiometricPage();
}

void MainWindow::update_label_width()
{
    for (int i = 0; i < label_name.count(); ++i) {
        label_name.at(i)->setFixedWidth((this->width() - 300) / 4 - 40);
    }
}

void MainWindow::on_listWidgetDevicesType_currentIndexChanged(int currentRow)
{
    int deviceType = (currentRow);
    QStringList headerData;
    headerData << "    " + tr("Name") << tr("Status") << tr("Default") << tr("Driver");
    m_listSwitchBtns.clear();
    ui->tableWidgetDevices->hide();
    ui->tableWidgetDevices->clear();
    ui->tableWidgetDevices->setRowCount(0);
    ui->tableWidgetDevices->setColumnCount(4);
    ui->tableWidgetDevices->setHorizontalHeaderLabels(headerData);
    //ui->tableWidgetDevices->setSelectionBehavior(QAbstractItemView::SelectRows); //设置选择行为，以行为单位
    ui->tableWidgetDevices->setSelectionMode(QAbstractItemView::NoSelection);

    ui->tableWidgetDevices->setFocusPolicy(Qt::NoFocus);

    for(int i = 0; i < headerData.size(); i++){
        if(i  == 0)
            ui->tableWidgetDevices->horizontalHeaderItem(i)->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
        else
            ui->tableWidgetDevices->horizontalHeaderItem(i)->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
    }

//    ui->tableWidgetDevices->setColumnWidth(0, 250);
//    ui->tableWidgetDevices->setColumnWidth(1, 152);
//    ui->tableWidgetDevices->setColumnWidth(2, 95);
//    ui->tableWidgetDevices->setColumnWidth(3, 143);
    ui->tableWidgetDevices->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);

    int column = 0;
    
    while(!btnGroup.isEmpty())
    {
           QRadioButton *box = btnGroup.last();
           btnGroup.removeLast();
           delete  box;
    }
    
    for(int i=0;i<deviceInfosMap[deviceType].count();i++)
    {
        if(deviceInfosMap[deviceType].at(i)->device_available > 0)
        {
            deviceInfosMap[deviceType].move(i,0);
        }
    }
    label_name.clear();
    for(auto deviceInfo : deviceInfosMap[deviceType]) {
        if(bioTypeToIndex(deviceInfo->biotype) == deviceType) {
            int row_index = ui->tableWidgetDevices->rowCount();
            if(column == 0)
                ui->tableWidgetDevices->insertRow(row_index);
            else
                row_index--;

            //第一 设备名称
            QWidget *item_name = new QWidget(ui->tableWidgetDevices);

//            item_name->setFlags(item_name->flags() ^ Qt::ItemIsEditable);
//            ui->tableWidgetDevices->setItem(row_index, column, item_name);
            QPushButton *labelIcon = new QPushButton(this);

            labelIcon->setObjectName("labelIcon");
            labelIcon->installEventFilter(this);
            labelIcon->setFlat(true);
            labelIcon->setEnabled(false);
            labelIcon->setMaximumSize(20,20);
            KALabel *labelName = new KALabel(this);
            QHBoxLayout *layout_name = new QHBoxLayout;
            layout_name->addWidget(labelIcon);
            labelIcon->setIcon(QIcon::fromTheme("preferences-system-details-symbolic"));
            layout_name->setProperty("useIconHighlightEffect", 0x8);

            layout_name->addWidget(labelName);
            labelName->setText(deviceInfo->device_shortname);
            labelName->setObjectName("deviceInfo->device_shortname");
            labelName->adjustSize();
            layout_name->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
            item_name->setLayout(layout_name);
            labelIcon->setProperty("devicename",deviceInfo->device_shortname);
            ui->tableWidgetDevices->setCellWidget(row_index, column, item_name);

            //第三 默认设备（是否设为默认设备）
            QWidget *item_default = new QWidget(ui->tableWidgetDevices);
            item_default->setObjectName("itemDefalut");
            QRadioButton *cbDefault = new QRadioButton(this);
            if(m_uniAuthService->getDefaultDevice(getenv("USER"), deviceInfo->biotype) == deviceInfo->device_shortname)
                cbDefault->setChecked(true);
            btnGroup.push_back(cbDefault);
            cbDefault->setObjectName(deviceInfo->device_shortname + "-" + QString::number(deviceInfo->biotype));
            connect(cbDefault, &QRadioButton::clicked, this, &MainWindow::onDefaultDeviceChanged);
            connect(m_uniAuthService, &UniAuthService::defaultDeviceChanged,
                    this, [&](const QString &userName, int bType,const QString &deviceName) {
                if(userName == getenv("USER")) {
                    for(auto btn : btnGroup)
                        btn->setChecked(false);

                    QString objName = deviceName+ "-" + QString::number(bType);
                    QRadioButton *check = findChild<QRadioButton*>(objName);
                    if(check) {
                        check->setChecked(true);
                    }
                }
            });
            if(cbDefault->isChecked()) {
                labelName->setText(tr("(default) ") + deviceInfo->device_shortname);
            } else
                labelName->setText(deviceInfo->device_shortname);
            label_name.append(labelName);
            QHBoxLayout *layout_default = new QHBoxLayout;
            layout_default->addWidget(cbDefault);
            layout_default->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
            item_default->setLayout(layout_default);
            ui->tableWidgetDevices->setCellWidget(row_index, column + 2, item_default);

            int flag = 0;

            //第四 驱动状态（是否使能）
            QWidget *item_drvStatus = new QWidget();
            KSwitchButton *btnDrvStatus = new KSwitchButton(this);
            //QPushButton *btnDrvStatus = new QPushButton(this);
            btnDrvStatus->setObjectName(deviceInfo->device_shortname + "_" + QString::number(deviceType));
            btnDrvStatus->setFixedSize(40, 20);
            if(deviceInfo->driver_enable > 0){
                btnDrvStatus->setChecked(true);
            }else{
                flag = 1 ;
                btnDrvStatus->setChecked(false);
            }
            connect(btnDrvStatus, &KSwitchButton::clicked, this, &MainWindow::onDriverStatusClicked);
            m_listSwitchBtns.append(btnDrvStatus);

            QVBoxLayout *layout = new QVBoxLayout(item_drvStatus);
            layout->addWidget(btnDrvStatus, 0, Qt::AlignVCenter | Qt::AlignHCenter);
            layout->setMargin(0);
            item_drvStatus->setLayout(layout);
            ui->tableWidgetDevices->setCellWidget(row_index, column + 3, item_drvStatus);

            //第二 设备状态（是否连接）
            QWidget *item_devStatus = new QWidget(ui->tableWidgetDevices);
            QHBoxLayout *layout_Status = new QHBoxLayout;
            KALabel *label = new KALabel(this);
            layout_Status->addWidget(label);
            KALabel *label1 = new KALabel(this);
            layout_Status->addWidget(label1);
            setDeviceStatus(label, label1, deviceInfo->device_available > 0, flag,
                            deviceInfo->biotype, deviceInfo->device_shortname);
            layout_Status->setAlignment(Qt::AlignVCenter | Qt::AlignHCenter);
            //layout_Status->setContentsMargins(20,0,0,0);
            item_devStatus->setLayout(layout_Status);
            ui->tableWidgetDevices->setCellWidget(row_index, column + 1, item_devStatus);
            ui->tableWidgetDevices->setRowHeight(row_index, 45);
//            column = (column + 4) % 4;
        }
    }
    ui->tableWidgetDevices->show();
}

void MainWindow::onDriverStatusClicked()
{
    QString objNameStr = sender()->objectName();
    KSwitchButton* switchBtn = qobject_cast<KSwitchButton*>(sender());
    qDebug() << objNameStr;
    int spliter = objNameStr.lastIndexOf('_');
    QString deviceName = objNameStr.left(spliter);
    int deviceType = objNameStr.right(objNameStr.length() - spliter - 1).toInt();

    qDebug() << deviceName << deviceType;

    if(deviceType != ui->listWidgetDevicesType->currentIndex())
        return;
    DeviceInfo *deviceInfo;
    for(auto info : deviceInfosMap[deviceType]) {
        if(info->device_shortname == deviceName) {
            deviceInfo = info;
            break;
        }
    }
    changeDeviceStatus(deviceInfo, switchBtn);
}

bool MainWindow::changeDeviceStatus(DeviceInfo *deviceInfo, KSwitchButton* switchBtn)
{
    if (!m_dialogLoading) {
        m_dialogLoading = new LoadingDialog(this, 0.4);
        m_dialogLoading->setGeometry(0,0, this->geometry().width(), this->geometry().height());
    } else {
        m_dialogLoading->setLoading();
    }
    m_dialogLoading->show();
    bool toEnable = deviceInfo->driver_enable <= 0 ? true : false;
    QString cmd;
    m_procEnableDriver = new QProcess(this);
    connect(m_procEnableDriver, static_cast<void(QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished),
            this, [&,this,toEnable,switchBtn](int exitCode, QProcess::ExitStatus status){
        qDebug()<<"Exit code:"<<exitCode;
        if (this->m_dialogLoading) {
            this->m_dialogLoading->hide();
            this->m_dialogLoading->close();
            update();
        }
        if (exitCode == 0) {
            this->lastDeviceInfo = deviceInfo;
            /*每次驱动状态改变都会改变服务状态，统一放入改变服务状态中处理*/
            //this->updateDevice();
        } else {
            QMessageBox criitical(QMessageBox::Critical,"",tr("Fail to change device status"));
            QPushButton *agreeBtn = criitical.addButton(tr("OK"), QMessageBox::AcceptRole);
            criitical.exec();
            if (switchBtn && this->m_listSwitchBtns.contains(switchBtn)) {
                switchBtn->setChecked(!toEnable);
            }
            update();
        }
        m_procEnableDriver->deleteLater();
    });

    if (toEnable) {
        cmd = "pkexec biodrvctl enable " //"pkexec biometric-config-tool enable-driver "
                + deviceInfo->device_shortname;
        qDebug() << cmd;
        m_procEnableDriver->start(cmd);
    } else {
        qDebug() << "disable" << deviceInfo->device_shortname;
        cmd = "pkexec biodrvctl disable " //"pkexec biometric-config-tool disable-driver "
                + deviceInfo->device_shortname;
        qDebug() << cmd;
        m_procEnableDriver->start(cmd);
    }
    return true;
}

bool MainWindow::restartService()
{
    if (!m_dialogLoading) {
        m_dialogLoading = new LoadingDialog(this, 0.4);
        m_dialogLoading->setGeometry(0,0, this->geometry().width(), this->geometry().height());
    } else {
        m_dialogLoading->setLoading();
    }
    m_dialogLoading->show();
    if (!m_procRestartService) {
        m_procRestartService = new QProcess(this);
        connect(m_procRestartService, static_cast<void(QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished),
                this, [&,this](int exitCode, QProcess::ExitStatus status){
            qDebug()<<"Exit code:"<<exitCode;
            if (this->m_dialogLoading) {
                this->m_dialogLoading->hide();
                //this->m_dialogLoading->close();
                update();
            }
            if (exitCode == 0) {
                QMessageBox criitical(QMessageBox::Information,"",tr("The service has restarted."));
                QPushButton *agreeBtn = criitical.addButton(tr("OK"), QMessageBox::AcceptRole);
                criitical.exec();
            } else {
                QMessageBox criitical(QMessageBox::Critical,"",tr("Service restart failure."));
                QPushButton *agreeBtn = criitical.addButton(tr("OK"), QMessageBox::AcceptRole);
                criitical.exec();
            }
        });
    }
    QString cmd = QString("pkexec biorestart");
    m_procRestartService->start(cmd);
    return true;
}

void MainWindow::updateDeviceListWidget(int biotype)
{
    QComboBox *lw = nullptr;

    switch(biotype) {
    case BIOTYPE_FINGERPRINT:
        lw = ui->listWidgetFingerPrint;
        break;
    case BIOTYPE_FINGERVEIN:
        lw = ui->listWidgetFingerVein;
        break;
    case BIOTYPE_IRIS:
        lw = ui->listWidgetIris;
        break;
    case BIOTYPE_FACE:
        lw = ui->listWidgetFace;
        break;
    case BIOTYPE_VOICEPRINT:
        lw = ui->listWidgetVoicePrint;
        break;
    }
    if(!lw) return;

    for(int row = 0; row < lw->count(); row++)
    {
        QList<DeviceInfo *> list = deviceInfosMap[bioTypeToIndex(biotype)];
        auto iter = std::find_if(list.begin(), list.end(),
                              [&](DeviceInfo *deviceInfo){
                return deviceInfo->device_shortname == lw->currentText(); });
////        if(iter != std::end(list))
////            item->setTextColor((*iter)->device_available ? Qt::black : Qt::gray);

    }

//    for(int row = 0; row < lw->count(); row++)
//    {
//        QListWidgetItem *item = lw->item(row);
//        QList<DeviceInfo *> list = deviceInfosMap[bioTypeToIndex(biotype)];
//        auto iter = std::find_if(list.begin(), list.end(),
//                              [&](DeviceInfo *deviceInfo){
//                return deviceInfo->device_shortname == item->text(); });
//        if(iter != std::end(list))
//            item->setTextColor((*iter)->device_available ? Qt::black : Qt::gray);

//    }
}

void MainWindow::updateDevice()
{
    //setCursor(Qt::WaitCursor);
    //qDebug() << "name:" << deviceInfo->device_shortname;

   //wait for service restart and dbus is ready
    QTimer::singleShot(3000, this, [&,this](){
        getDeviceInfo();
        on_listWidgetDevicesType_currentIndexChanged(ui->listWidgetDevicesType->currentIndex());

       /* for(int i = 0; i < __MAX_NR_BIOTYPES; i++){
        for(auto deviceInfo : deviceInfosMap[i]){
                ContentPane *contentPane = contentPaneMap[deviceInfo->device_shortname];
                if(contentPane){
                    contentPane->setDeviceInfo(deviceInfo);
                    contentPane->showFeatures();
                }else{
                    addContentPane(deviceInfo);
                }
        }
        updateDeviceListWidget(i);
        }*/
        initBiometricPage();
        //setCursor(Qt::ArrowCursor);
        //sortContentPane();
    });
}

void MainWindow::on_tableWidgetDevices_cellDoubleClicked(int row, int column)
{
    if(column % 4 != 0)
        return;
    int index = row ;

    if(index < deviceInfosMap[ui->listWidgetDevicesType->currentIndex()].size()) {
        int deviceType = ui->listWidgetDevicesType->currentIndex();
        DeviceInfo *deviceInfo =  deviceInfosMap[deviceType][index];
        if(deviceInfo->device_available > 0){
            QComboBox *lw;
            switch(deviceInfo->biotype) {
            case BIOTYPE_FINGERPRINT:
                lw = ui->listWidgetFingerPrint;
                onBtnFingerPrintClicked();
                break;
            case BIOTYPE_FINGERVEIN:
                lw = ui->listWidgetFingerVein;
                onBtnFingerVeinClicked();
                break;
            case BIOTYPE_IRIS:
                lw = ui->listWidgetIris;
                onBtnIrisClicked();
                break;
            case BIOTYPE_FACE:
                lw = ui->listWidgetFace;
                onBtnVoicePrintClicked();
                break;
            case BIOTYPE_VOICEPRINT:
                lw = ui->listWidgetVoicePrint;
                onBtnVoicePrintClicked();
                break;
            }
            lw->setCurrentIndex(index);
        }
    }
}

void MainWindow::onUpdateDevice(int drvid,int uid,int cType)
{
    usleep(500000);  //延迟500ms，否则当控制面板进行重命名时，会出现设备忙导致，指纹列表为空
    for(int type : deviceInfosMap.keys()) {
        auto &deviceInfoList = deviceInfosMap[type];
        for(int i = 0; i < deviceInfoList.size(); i++) {
            auto deviceInfo = deviceInfoList[i];
            if(deviceInfo->device_id == drvid) {
                if (contentPaneMap.contains(deviceInfo->device_shortname)) {
                    ContentPane *pane = contentPaneMap[deviceInfo->device_shortname];
                    if(pane && !pane->getIsShowDialog()){
                        pane->showFeatures();
                    }
                }
            }
        }
    }
}

void MainWindow::onUSBDeviceHotPlug(int drvid, int action, int devNumNow)
{
    qDebug() << "device"<< (action > 0 ? "insert:" : "pull out:");
    qDebug() << "id:" << drvid;
    sleep(1);
    for(int type : deviceInfosMap.keys()) {
        auto &deviceInfoList = deviceInfosMap[type];
        for(int i = 0; i < deviceInfoList.size(); i++) {
            auto deviceInfo = deviceInfoList[i];
            if(deviceInfo->device_id == drvid) {
                qDebug() << "name:" << deviceInfo->device_shortname;
                //更新结构体
                deviceInfo->device_available = devNumNow;
                //更新标签页中的设备状态
                clearDeviceNum();
                getDeviceInfo();
                ContentPane *pane = contentPaneMap[deviceInfo->device_shortname];
                updateBiometricPage(deviceInfo->biotype ,getDeviceNum(deviceInfo->biotype));
                on_listWidgetDevicesType_currentIndexChanged(ui->listWidgetDevicesType->currentIndex());
                initBiometricPage();
                return;
            }
        }
    }
}

void MainWindow::tabletStatusChanged(bool tablet_mode)
{
    if(tablet_mode) {
        ui->btnMax->hide();
        ui->widgetHeader_3->setContentsMargins(8, 8, 0, 0);
        ui->lblLogo->setFixedSize(32, 32);
        ui->lblLogo->setPixmap(QIcon::fromTheme("biometric-manager").pixmap(QSize(32, 32)));
        ui->widgetHeader_3->setFixedHeight(64);
        ui->widgetHeader->setContentsMargins(8, 8, 8, 14);
        ui->btnMenu->setFixedSize(48, 48);
        ui->btnClose->setFixedSize(48, 48);
        ui->btnMin->setFixedSize(48, 48);
    } else {
        ui->btnMax->show();
        ui->widgetHeader_3->setContentsMargins(0, 0, 0, 0);
        ui->widgetHeader_3->setFixedHeight(46);
        ui->lblLogo->setPixmap(QIcon::fromTheme("biometric-manager").pixmap(QSize(24, 24)));
        ui->lblLogo->setFixedSize(24, 24);
        ui->widgetHeader->setContentsMargins(8, 4, 4, 14);
        ui->btnMenu->setFixedSize(30, 30);
        ui->btnClose->setFixedSize(30, 30);
        ui->btnMin->setFixedSize(30, 30);
    }
}

void MainWindow::setDeviceStatus(KALabel *iconLabel, KALabel *textLabel, bool connected, int flag, int bioType, QString defaultName)
{
    QString deviceStatus;
    iconLabel->setFixedSize(8, 8);
    if(connected && flag == 0) {
        deviceStatus = tr("Connected");
        iconLabel->setStyleSheet("background: #4DF46D; border-radius: 4px; opacity: 1;");
        //如果设备状态为已连接且当前设备类型未设置默认设备，直接将该设备设为默认设备
        if( m_uniAuthService->getDefaultDevice(getenv("USER"), bioType) == "") {
            qDebug() << "设备首次接入未设置默认设备，type : " << bioType << "defaultName :" << defaultName;
            m_uniAuthService->setDefaultDevice(bioType, defaultName);
            //设置默认设备后刷新设备列表
            on_listWidgetDevicesType_currentIndexChanged(ui->listWidgetDevicesType->currentIndex());
            initBiometricPage();
        }
    }
    else if(!connected && flag == 0){
        deviceStatus = tr("Unconnected");
        iconLabel->setStyleSheet("background: #939393; border-radius: 4px; opacity: 1;");
    }
    else if(flag == 1){
        deviceStatus = tr("disable");
        iconLabel->setStyleSheet("background: #FD1C1C; border-radius: 4px; opacity: 1;");
    }
    textLabel->setText(deviceStatus);
}


void MainWindow::onServiceStatusChanged(bool activate)
{
    if(!activate)
    {
        // 关闭子窗口
        QMap<QString, ContentPane *>::iterator itContentPane = contentPaneMap.begin();
        for (; itContentPane != contentPaneMap.end(); itContentPane++) {
            ContentPane *pane = itContentPane.value();
            if (pane) {
                pane->clearSubWidgets();
            }
        }
        if (!m_dialogLoading) {
            m_dialogLoading = new LoadingDialog(this, 0.4);
            m_dialogLoading->setGeometry(0,0, this->geometry().width(), this->geometry().height());
        } else {
            m_dialogLoading->setLoading();
        }
        m_dialogLoading->show();
    }
    else
    {
        qDebug()<<"onServiceStatusChanged :service restarted!";
        //updateDevice();
        QTimer::singleShot(1500, this, [&,this](){
            getDeviceInfo();
            on_listWidgetDevicesType_currentIndexChanged(ui->listWidgetDevicesType->currentIndex());
            initBiometricPage();
            if (this->m_dialogLoading) {
                m_dialogLoading->hide();
                //this->m_dialogLoading->close();
                update();
            }
        });
    }
}

void MainWindow::paintEvent(QPaintEvent *)
{
    QPainterPath path;

    QPainter painter(this);
    painter.setOpacity(1);
    painter.setRenderHint(QPainter::Antialiasing);  // 反锯齿;
    painter.setClipping(true);
    painter.setPen(Qt::transparent);

    path.addRoundedRect(240, 0 ,ui->widget->width(), ui->widget->width(), 6, 6);
    path.setFillRule(Qt::WindingFill);
    painter.setBrush(this->palette().base());
    painter.setPen(Qt::transparent);

    painter.drawPath(path);
    QStyleOption opt;
    opt.init(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &painter, this);
}

void MainWindow::setStackedWidgetMain(QWidget* newWidget)
{
    ui->stackedWidgetMain->setCurrentWidget(newWidget);
    isdashBoardWidget = true;
    isPermissionWidget = false;
    scrollArea->listview()->setCurrentIndex(dashBoardItem->index());
    //newWidget->adjustSize();
    update();
}
