#include "changefeaturename.h"
#include "ui_changefeaturename.h"
#include "ukuistylehelper/ukuistylehelper.h"
#include "windowmanager/windowmanager.h"
#include <QDebug>


extern void qt_blurImage(QImage &blurImage, qreal radius, bool quality, int transposed);

ChangeFeatureName::ChangeFeatureName(QStringList featurenames, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ChangeFeatureName),
    names(featurenames)
{
    //去除窗管标题栏
    kdk::UkuiStyleHelper::self()->removeHeader(this);
    ui->setupUi(this);
    setAttribute(Qt::WA_DeleteOnClose);

    ui->logoLabel->setPixmap(QIcon::fromTheme("ukui-control-center").pixmap(QSize(24,24)));
    ui->closeBtn->setToolTip(tr("Close"));
    ui->closeBtn->setProperty("isWindowButton", 0x02);
    ui->closeBtn->setProperty("useIconHighlightEffect", 0x08);
    ui->closeBtn->setFlat(true);
    ui->closeBtn->setIcon(QIcon::fromTheme("window-close-symbolic"));
    connect(ui->closeBtn, &QPushButton::clicked, this, &ChangeFeatureName::close);
    ui->saveBtn->setEnabled(false);
    ui->lineEdit->setFocus();

    connect(ui->lineEdit, &QLineEdit::textChanged, this, [=](QString txt){
        if (names.contains(txt)){
            ui->tipLabel->setText(tr("Duplicate feature name"));
        } else if(txt.startsWith(" ") || txt.endsWith(" ")){
            ui->tipLabel->setText(tr("Please do not use spaces as the beginning or end of the feature name"));
        } else if(txt.isEmpty()) {
            ui->tipLabel->setText(tr("Empty feature name"));
        } else if (txt.length() > 64) {
            ui->tipLabel->setText(tr("feature name is too long"));
        } else {
            ui->tipLabel->setText(tr(""));
        }
        if (!txt.isEmpty() && ui->tipLabel->text().isEmpty()){
            ui->saveBtn->setEnabled(true);
        } else {
            ui->saveBtn->setEnabled(false);
        }
    });
    connect(ui->cancelBtn, &QPushButton::clicked, [=]{
        close();
    });
    connect(ui->saveBtn, &QPushButton::clicked, [=]{
        emit sendNewName(ui->lineEdit->text());
        close();
    });
}

ChangeFeatureName::~ChangeFeatureName()
{
    delete ui;
}

void ChangeFeatureName::keyPressEvent(QKeyEvent * event){
    switch (event->key())
    {
    case Qt::Key_Escape:
        break;
    case Qt::Key_Enter:
        if (ui->saveBtn->isEnabled())
            ui->saveBtn->clicked();
        break;
    case Qt::Key_Return:
        if (ui->saveBtn->isEnabled())
            ui->saveBtn->clicked();
        break;
    default:
        QDialog::keyPressEvent(event);
    }
}

void ChangeFeatureName::setTitle(QString titleName){
    ui->titleLabel->setText(titleName + tr(" Rename"));
    ui->renameLabel->setText(titleName + tr(" name"));
    QFont font;
    QString str = ElideText(font,95,ui->renameLabel->text());
    if(str != ui->renameLabel->text()){
        ui->renameLabel->setToolTip(ui->renameLabel->text());
        ui->renameLabel->setText(str);
    }
}

QString ChangeFeatureName::ElideText(QFont font,int width,QString strInfo)
{
    QFontMetrics fontMetrics(font);
    //如果当前字体下，字符串长度大于指定宽度
    if(fontMetrics.width(strInfo) > width)
    {
        strInfo= QFontMetrics(font).elidedText(strInfo, Qt::ElideRight, width);
    }
    return strInfo;
}
